; A simple program to test free running timer in timer module.
; uses interrupts
;
; DESCRIPTION:
;  The program handles TOF interrupts which occur every 0x10000 (65,536)
;  bus cycles.  If the bus cycle is 1 microsecond (i.e. a 4MHz clock),
;  interrupts occur about 16 times a second.
;
;  The interrupt second routine (isr) maintains a "sixteenth of a
;  second" down counter (sec16 at address 0x7000) which is decremented
;  at each interrupt.  When it counts down to zero, the "seconds
;  counter"  (secs at address 0x7001) is incremented and sec16 is
;  reset to 16 (0x10).
;
;  In short, the contents of address 0x7000 are decremented each 1/16
;  second and the contents of address 0x7001 are incremented each
;  second.
;
; USAGE NOTES:
; Try the following:
;   1) Set a breakpoint at the beginning of the interrupt service
;      routine.
;   2) Set Viewpoints for addresses 0x7000 (sec16) and 0x7001 (secs).
;   3) Run the program at full speed.  The interrupt breakpoint should
;      be hit quickly.  The viewpoint at 0x7000 should have a smaller
;      value.
;   4) Keep on hitting the RUN button; after 16 times, the viewpoint
;      at 0x7001 (seconds) should increment to "01".  The statistics
;      should also indicate that about one million bus cycles have been
;      simulated.
;
; Now try this:
;   1) Remove the breakpoint.
;   2) Run at full speed.
;   3) Both viewpoints should now change (quite rapidly).
;
; PERFORMANCE MEASUREMENTS:
; 
;  This simple program exercises many of the simulator's internal
;  capabilities.  Consequently, it is a reasonable test of how an
;  implementation  of the simulator on specific hardware and operating
;  system compares with a real 6811 CPU.
;
;  One simple test is to measure (with an ordinary wrist watch) the
;  time it takes to simulate (approximately) 256 seconds.  To do this,
;  note the time it takes for the seconds counter (at 0x7001) to cycle
;  between 00 and back to 00 again.
;
;  On my laptop, for example, it takes about 40 seconds to do
;  this.  Hence, the simulation is about 6.5 times faster than a real
;  6811.
;
;  This measure gives an overall metric of the simulator's efficiency.
;  However, it also includes GUI overhead such as updating the 0x7000
;  viewpoint at every interrupt.  If this veiwpoint is eliminated,
;  better performance is achieved.  On my laptop, I get about 32
;  seconds indicating that the simulation engine is at least 8 times
;  faster than a real 6811.
;
;  (Note: the instruction mix also influences this metric.  For
;  example, if the main infinite loop does an "idiv" instruction
;  instead of a "nop", it takes less then 20 seconds to simulate
;  256 seconds--a speedup of a factor of 13.)
;
;    PERFORMANCE TECHNICAL DETAILS:
;
;                   SUMMARY OF METRICS
;             Test                        Speedup (compared to real 6811)
;        Viewpoints 7000/7001 nop loop           6.5
;        Viewpoints 7001 nop loop                8
;        Viewpoints 7001 idiv loop              13
;
;     The numbers used here are based on:
;
;          Hardware:
;            * 800 MHz DURON processor (the AMD DURON
;              is roughly equivalent to an INTEL Celeron);
;
;            * 128 MB RAM
;
;          Software:
;            * Windows ME Operating System
;            * Java VM 1.4.1 using -server and -ea flags
;            * sim11 Version 0.96
;
;     Let me know if you get significantly different
;     results (email: kclowes@ee.ryerson.ca).
;
;       (minor P.S.:  using IBM's "jikes" compiler instead
;       of Sun's "javac" seems to reduce class sizes by
;       about 2-3% and also to reduce running times by about
;       0.5%...  More importantly, "jikes" has more
;       informative error messages and capabilities.
;       It also compiles 200-400% faster...
;       Highly recommended!)
; 
REG_BAS equ $1000
TCNT_OFFSET equ $0e
TCNT_ABS equ REG_BAS+TCNT_OFFSET
TFLG2_OFFSET equ $25
TFLG2_ABS equ REG_BAS+TFLG2_OFFSET
TMSK2_OFFSET equ $24
TMSK2_ABS equ REG_BAS+TMSK2_OFFSET

;  This program runs under either a "pure" 6811 simulation
;  environment OR under the soft interrupt vectors as defined
;  by the BUFFALO monitor.

;  The simulator by default loads the Buffalo monitor vectors.
;  Hence, MOST USERS should use the Buffalo monitor soft vectors as
;  defined below.
;
;  IF you are burning custom reset routines and hard vectors into ROM
;  uncomment the soft vector definition of TOF_VECTOR.

*TOF_VECTOR equ $FFDE  ;Uncomment if running on "pure" 6811
TOF_VECTOR equ $00D1   ;Uncomment if Buffalo monitor soft vectors used


      org $6000
main:
      sei ;disable interrupts during set-up
      jsr init
      cli ;enable interrupts

forever:
        nop
        bra forever

init:
  	clr secs
	ldaa #16
        staa sec16
 	ldaa #$80
        staa TMSK2_ABS
        rts

isr:
        ldaa #$80
        staa TFLG2_ABS
        dec sec16
        bne isr_continue
        ldaa #16
        staa sec16
        inc secs
isr_continue:
	rti	

        org $7000 ; RAM segment
sec16 rmb 1
secs  rmb 1

	org TOF_VECTOR
	fdb isr
           end
